# pylint: disable=missing-docstring, too-few-public-methods, pointless-statement


class Parent(object):

    def __init__(self):
        self._parent = 42
        self._registry = {}
        self._similar1 = 24
        self._similar2 = 23
        self._similar3 = 24
        self._really_similar1 = 23
        self._really_similar2 = 42


class Child(Parent):

    def __init__(self):
        super(Child, self).__init__()

        self._similar # [no-member]
        self._really_similar # [no-member]
        self._paren # [no-member]
        # Distance is too big
        self._registryyyy # [no-member]
        # Nothing close.
        self._pretty_sure_this_wont_match # [no-member]



# [missing-docstring]
# pylint: disable=too-few-public-methods

def public_documented():
    """It has a docstring."""


def _private_undocumented():
    # Doesn't need a docstring
    pass


def _private_documented():
    """It has a docstring."""


class ClassDocumented(object):
    """It has a docstring."""


class ClassUndocumented(object): # [missing-docstring]
    pass


def public_undocumented(): # [missing-docstring]
    pass


def __sizeof__():
    # Special
    pass


def __mangled():
    pass


class Property(object):
    """Don't warn about setters and deleters."""

    def __init__(self):
        self._value = None

    @property
    def test(self):
        """Default docstring for setters and deleters."""

    @test.setter
    def test(self, value):
        self._value = value

    @test.deleter
    def test(self):
        pass



"""Contains both normal error messages and Python3 porting error messages."""
# pylint: disable=too-few-public-methods

# error: import missing `from __future__ import absolute_import`
import sys

# error: Use raise ErrorClass(args) instead of raise ErrorClass, args.
raise Exception, 1

class Test(object):
    """dummy"""

    def __init__(self):
        # warning: Calling a dict.iter*() method
        {1: 2}.iteritems()
        return 42

# error: print statement used
print 'not in python3'



'''
Regression test for
https://bitbucket.org/logilab/pylint/issue/128/attributeerror-when-parsing
'''
from __future__ import with_statement

def do_nothing():
    """ empty """
    with open("") as ctx.obj:  # [undefined-variable]
        context.do()  # [used-before-assignment]
        context = None



"""Warnings for using type(x) == Y or type(x) is Y instead of isinstance(x, Y)."""
# pylint: disable=missing-docstring,expression-not-assigned,redefined-builtin,invalid-name

def simple_positives():
    type(42) is int # [unidiomatic-typecheck]
    type(42) is not int # [unidiomatic-typecheck]
    type(42) == int # [unidiomatic-typecheck]
    type(42) != int # [unidiomatic-typecheck]
    type(42) in [int] # [unidiomatic-typecheck]
    type(42) not in [int] # [unidiomatic-typecheck]

def simple_inference_positives():
    alias = type
    alias(42) is int # [unidiomatic-typecheck]
    alias(42) is not int # [unidiomatic-typecheck]
    alias(42) == int # [unidiomatic-typecheck]
    alias(42) != int # [unidiomatic-typecheck]
    alias(42) in [int] # [unidiomatic-typecheck]
    alias(42) not in [int] # [unidiomatic-typecheck]

def type_creation_negatives():
    type('Q', (object,), dict(a=1)) is int
    type('Q', (object,), dict(a=1)) is not int
    type('Q', (object,), dict(a=1)) == int
    type('Q', (object,), dict(a=1)) != int
    type('Q', (object,), dict(a=1)) in [int]
    type('Q', (object,), dict(a=1)) not in [int]

def invalid_type_call_negatives(**kwargs):
    type(bad=7) is int
    type(bad=7) is not int
    type(bad=7) == int
    type(bad=7) != int
    type(bad=7) in [int]
    type(bad=7) not in [int]
    type('bad', 7) is int
    type('bad', 7) is not int
    type('bad', 7) == int
    type('bad', 7) != int
    type('bad', 7) in [int]
    type('bad', 7) not in [int]
    type(**kwargs) is int
    type(**kwargs) is not int
    type(**kwargs) == int
    type(**kwargs) != int
    type(**kwargs) in [int]
    type(**kwargs) not in [int]

def local_var_shadowing_inference_negatives():
    type = lambda dummy: 7
    type(42) is int
    type(42) is not int
    type(42) == int
    type(42) != int
    type(42) in [int]
    type(42) not in [int]

def parameter_shadowing_inference_negatives(type):
    type(42) is int
    type(42) is not int
    type(42) == int
    type(42) != int
    type(42) in [int]
    type(42) not in [int]

def deliberate_subclass_check_negatives(b):
    type(42) is type(b)
    type(42) is not type(b)

def type_of_literals_positives(a):
    type(a) is type([]) # [unidiomatic-typecheck]
    type(a) is not type([]) # [unidiomatic-typecheck]
    type(a) is type({}) # [unidiomatic-typecheck]
    type(a) is not type({}) # [unidiomatic-typecheck]
    type(a) is type("") # [unidiomatic-typecheck]
    type(a) is not type("") # [unidiomatic-typecheck]



"""Check for definitions and usage happening on the same line."""
#pylint: disable=missing-docstring,multiple-statements,no-absolute-import,parameter-unpacking,wrong-import-position
from __future__ import print_function
print([index
       for index in range(10)])

print((index
       for index in range(10)))

FILTER_FUNC = lambda x: not x

def func(xxx): return xxx

def func2(xxx): return xxx + func2(1)

import sys; print(sys.exc_info())

for i in range(10): print(i)

j = 4; LAMB = lambda x: x+j

FUNC4 = lambda a, b: a != b

# test http://www.logilab.org/ticket/6954:

with open('f') as f: print(f.read())

with open('f') as f, open(f.read()) as g:
    print(g.read())



# pylint: disable=missing-docstring,too-few-public-methods,import-error
from UNINFERABLE import ImportedMetaclass


class Meta(type):
    pass


class Class(metaclass=Meta):
    pass


def func_scope():
    class Meta2(type):
        pass

    class Class2(metaclass=Meta2):
        pass

    return Class2


class ClassScope:
    class Meta3(type):
        pass

    class Class3(metaclass=Meta3):
        pass

    instance = Class3()


def mixed_scopes():
    class ClassM(metaclass=Meta):
        pass

    return ClassM


def imported_and_nested_scope1():
    class ClassImp1(metaclass=ImportedMetaclass):
        pass

    class ClassImp2(metaclass=ImportedMetaclass):
        pass

    return ClassImp1, ClassImp2


def imported_and_nested_scope2():
    from UNINFERABLE import ImportedMetaclass2

    class ClassImp3(metaclass=ImportedMetaclass2):
        pass

    return ClassImp3



# Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2010 Julien Jehannet <julien.jehannet@logilab.fr>
# Copyright (c) 2013-2014 Google, Inc.
# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2014 Brett Cannon <brett@python.org>
# Copyright (c) 2014 Ricardo Gemignani <ricardo.gemignani@gmail.com>
# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
# Copyright (c) 2016 Florian Bruhin <git@the-compiler.org>
# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
# Copyright (c) 2017 Hugo <hugovk@users.noreply.github.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

# pylint: disable=W0622,C0103
"""pylint packaging information"""

from __future__ import absolute_import

from os.path import join
from sys import version_info as py_version

from pkg_resources import parse_version
from setuptools import __version__ as setuptools_version

modname = distname = 'pylint'

numversion = (1, 9, 2)
version = '.'.join([str(num) for num in numversion])

install_requires = [
    'astroid>=1.6,<2.0',
    'six',
    'isort >= 4.2.5',
    'mccabe',
]

dependency_links = []

extras_require = {}
extras_require[':sys_platform=="win32"'] = ['colorama']


def has_environment_marker_range_operators_support():
    """Code extracted from 'pytest/setup.py'
    https://github.com/pytest-dev/pytest/blob/7538680c/setup.py#L31
    The first known release to support environment marker with range operators
    it is 17.1, see: https://setuptools.readthedocs.io/en/latest/history.html#id113
    """
    return parse_version(setuptools_version) >= parse_version('17.1')


if has_environment_marker_range_operators_support():
    extras_require[':python_version=="2.7"'] = ['configparser', 'backports.functools_lru_cache']
    extras_require[':python_version<"3.4"'] = ['singledispatch']
else:
    if (py_version.major, py_version.minor) == (2, 7):
        install_requires.extend(['configparser', 'backports.functools_lru_cache'])
    if py_version < (3, 4):
        install_requires.extend(['singledispatch'])


license = 'GPL'
description = "python code static checker"
web = 'https://github.com/PyCQA/pylint'
mailinglist = "mailto:code-quality@python.org"
author = 'Python Code Quality Authority'
author_email = 'code-quality@python.org'

classifiers = ['Development Status :: 4 - Beta',
               'Environment :: Console',
               'Intended Audience :: Developers',
               'License :: OSI Approved :: GNU General Public License (GPL)',
               'Operating System :: OS Independent',
               'Programming Language :: Python',
               'Programming Language :: Python :: 2',
               'Programming Language :: Python :: 2.7',
               'Programming Language :: Python :: 3',
               'Programming Language :: Python :: 3.4',
               'Programming Language :: Python :: 3.5',
               'Programming Language :: Python :: 3.6',
               'Programming Language :: Python :: Implementation :: CPython',
               'Programming Language :: Python :: Implementation :: PyPy',
               'Topic :: Software Development :: Debuggers',
               'Topic :: Software Development :: Quality Assurance',
               'Topic :: Software Development :: Testing'
              ]


long_desc = """\
 Pylint is a Python source code analyzer which looks for programming
 errors, helps enforcing a coding standard and sniffs for some code
 smells (as defined in Martin Fowler's Refactoring book)
 .
 Pylint can be seen as another PyChecker since nearly all tests you
 can do with PyChecker can also be done with Pylint. However, Pylint
 offers some more features, like checking length of lines of code,
 checking if variable names are well-formed according to your coding
 standard, or checking if declared interfaces are truly implemented,
 and much more.
 .
 Additionally, it is possible to write plugins to add your own checks.
 .
 Pylint is shipped with "pyreverse" (UML diagram generator)
 and "symilar" (an independent similarities checker)."""

scripts = [join('bin', filename)
           for filename in ('pylint', "symilar", "epylint",
                            "pyreverse")]

include_dirs = [join('pylint', 'test')]



"""Logging warnings using a logger class."""
from __future__ import absolute_import
import logging

__revision__ = ''

LOG = logging.getLogger("domain")
LOG.debug("%s" % "junk")
LOG.log(logging.DEBUG, "%s" % "junk")
LOG2 = LOG.debug
LOG2("%s" % "junk")

logging.getLogger("domain").debug("%s" % "junk")



# -*- coding: utf-8 -*-
# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2014-2017 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2014 Vlad Temian <vladtemian@gmail.com>
# Copyright (c) 2014 Google, Inc.
# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
# Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>
# Copyright (c) 2017 Bryce Guinta <bryce.paul.guinta@gmail.com>
# Copyright (c) 2017 Thomas Hisch <t.hisch@gmail.com>
# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import contextlib
import json
import re
import sys
import os
from os.path import join, dirname, abspath
import tempfile
import textwrap

import six

from pylint.lint import Run
from pylint.reporters import BaseReporter
from pylint.reporters.text import *
from pylint.reporters.json import JSONReporter
import pytest
from pylint import utils

HERE = abspath(dirname(__file__))



@contextlib.contextmanager
def _patch_streams(out):
    sys.stderr = sys.stdout = out
    try:
        yield
    finally:
        sys.stderr = sys.__stderr__
        sys.stdout = sys.__stdout__


@contextlib.contextmanager
def _configure_lc_ctype(lc_ctype):
    lc_ctype_env = 'LC_CTYPE'
    original_lctype = os.environ.get(lc_ctype_env)
    os.environ[lc_ctype_env] = lc_ctype
    try:
        yield
    finally:
        os.environ.pop(lc_ctype_env)
        if original_lctype:
            os.environ[lc_ctype_env] = original_lctype


class MultiReporter(BaseReporter):
    def __init__(self, reporters):
        self._reporters = reporters
        self.path_strip_prefix = os.getcwd() + os.sep

    def on_set_current_module(self, *args, **kwargs):
        for rep in self._reporters:
            rep.on_set_current_module(*args, **kwargs)

    def handle_message(self, msg):
        for rep in self._reporters:
            rep.handle_message(msg)

    def display_reports(self, layout):
        pass

    @property
    def out(self):
        return self._reporters[0].out

    @property
    def linter(self):
        return self._linter

    @linter.setter
    def linter(self, value):
        self._linter = value
        for rep in self._reporters:
            rep.linter = value


class TestRunTC(object):

    def _runtest(self, args, reporter=None, out=None, code=28):
        if out is None:
            out = six.StringIO()
        pylint_code = self._run_pylint(args, reporter=reporter, out=out)
        if reporter:
            output = reporter.out.getvalue()
        elif hasattr(out, 'getvalue'):
            output = out.getvalue()
        else:
            output = None
        msg = 'expected output status %s, got %s' % (code, pylint_code)
        if output is not None:
            msg = '%s. Below pylint output: \n%s' % (msg, output)
        assert pylint_code == code, msg

    def _run_pylint(self, args, out, reporter=None):
        args = args + ['--persistent=no']
        with _patch_streams(out):
            with pytest.raises(SystemExit) as cm:
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    Run(args, reporter=reporter)
            return cm.value.code

    def _test_output(self, args, expected_output):
        out = six.StringIO()
        self._run_pylint(args, out=out)
        actual_output = out.getvalue()
        assert expected_output.strip() in actual_output.strip()

    def test_pkginfo(self):
        """Make pylint check itself."""
        self._runtest(['pylint.__pkginfo__'], reporter=TextReporter(six.StringIO()),
                      code=0)

    def test_all(self):
        """Make pylint check itself."""
        reporters = [
            TextReporter(six.StringIO()),
            ColorizedTextReporter(six.StringIO()),
            JSONReporter(six.StringIO())
        ]
        self._runtest(['pylint/test/functional/arguments.py'],
                      reporter=MultiReporter(reporters), code=1)

    def test_no_ext_file(self):
        self._runtest([join(HERE, 'input', 'noext')], code=0)

    def test_w0704_ignored(self):
        self._runtest([join(HERE, 'input', 'ignore_except_pass_by_default.py')], code=0)

    def test_generate_config_option(self):
        self._runtest(['--generate-rcfile'], code=0)

    def test_generate_config_option_order(self):
        out1 = six.StringIO()
        out2 = six.StringIO()
        self._runtest(['--generate-rcfile'], code=0, out=out1)
        self._runtest(['--generate-rcfile'], code=0, out=out2)
        output1 = out1.getvalue()
        output2 = out2.getvalue()
        assert output1 == output2

    def test_generate_config_disable_symbolic_names(self):
        # Test that --generate-rcfile puts symbolic names in the --disable
        # option.

        out = six.StringIO()
        self._run_pylint(["--generate-rcfile", "--rcfile="], out=out)

        output = out.getvalue()
        # Get rid of the pesky messages that pylint emits if the
        # configuration file is not found.
        master = re.search(r"\[MASTER", output)
        out = six.StringIO(output[master.start():])
        parser = six.moves.configparser.RawConfigParser()
        parser.readfp(out)
        messages = utils._splitstrip(parser.get('MESSAGES CONTROL', 'disable'))
        assert 'suppressed-message' in messages

    def test_generate_rcfile_no_obsolete_methods(self):
        out = six.StringIO()
        self._run_pylint(["--generate-rcfile"], out=out)
        output = out.getvalue()
        assert "profile" not in output

    def test_inexisting_rcfile(self):
        out = six.StringIO()
        with pytest.raises(IOError) as excinfo:
            self._run_pylint(["--rcfile=/tmp/norcfile.txt"], out=out)
        assert "The config file /tmp/norcfile.txt doesn't exist!" == str(excinfo.value)

    def test_help_message_option(self):
        self._runtest(['--help-msg', 'W0101'], code=0)

    def test_error_help_message_option(self):
        self._runtest(['--help-msg', 'WX101'], code=0)

    def test_error_missing_arguments(self):
        self._runtest([], code=32)

    def test_no_out_encoding(self):
        """test redirection of stdout with non ascii caracters
        """
        #This test reproduces bug #48066 ; it happens when stdout is redirected
        # through '>' : the sys.stdout.encoding becomes then None, and if the
        # output contains non ascii, pylint will crash
        if sys.version_info < (3, 0):
            strio = tempfile.TemporaryFile()
        else:
            strio = six.StringIO()
        assert strio.encoding is None
        self._runtest([join(HERE, 'regrtest_data/no_stdout_encoding.py'),
                       '--enable=all'],
                      out=strio, code=28)

    def test_parallel_execution(self):
        self._runtest(['-j 2', 'pylint/test/functional/arguments.py',
                       'pylint/test/functional/bad_continuation.py'], code=1)

    def test_parallel_execution_missing_arguments(self):
        self._runtest(['-j 2', 'not_here', 'not_here_too'], code=1)

    def test_py3k_option(self):
        # Test that --py3k flag works.
        rc_code = 2 if six.PY2 else 0
        self._runtest([join(HERE, 'functional', 'unpacked_exceptions.py'),
                       '--py3k'],
                      code=rc_code)

    def test_py3k_jobs_option(self):
        rc_code = 2 if six.PY2 else 0
        self._runtest([join(HERE, 'functional', 'unpacked_exceptions.py'),
                       '--py3k', '-j 2'],
                      code=rc_code)

    @pytest.mark.skipif(sys.version_info[0] > 2, reason="Requires the --py3k flag.")
    def test_py3k_commutative_with_errors_only(self):

        # Test what gets emitted with -E only
        module = join(HERE, 'regrtest_data', 'py3k_error_flag.py')
        expected = textwrap.dedent("""
        ************* Module py3k_error_flag
        Explicit return in __init__
        """)
        self._test_output([module, "-E", "--msg-template='{msg}'"],
                          expected_output=expected)

        # Test what gets emitted with -E --py3k
        expected = textwrap.dedent("""
        ************* Module py3k_error_flag
        Use raise ErrorClass(args) instead of raise ErrorClass, args.
        """)
        self._test_output([module, "-E", "--py3k", "--msg-template='{msg}'"],
                          expected_output=expected)

        # Test what gets emitted with --py3k -E
        self._test_output([module, "--py3k", "-E", "--msg-template='{msg}'"],
                          expected_output=expected)

    @pytest.mark.skipif(sys.version_info[0] > 2, reason="Requires the --py3k flag.")
    def test_py3k_commutative_with_config_disable(self):
        module = join(HERE, 'regrtest_data', 'py3k_errors_and_warnings.py')
        rcfile = join(HERE, 'regrtest_data', 'py3k-disabled.rc')
        cmd = [module, "--msg-template='{msg}'", "--reports=n"]

        expected = textwrap.dedent("""
        ************* Module py3k_errors_and_warnings
        import missing `from __future__ import absolute_import`
        Use raise ErrorClass(args) instead of raise ErrorClass, args.
        Calling a dict.iter*() method
        print statement used
        """)
        self._test_output(cmd + ["--py3k"], expected_output=expected)

        expected = textwrap.dedent("""
        ************* Module py3k_errors_and_warnings
        Use raise ErrorClass(args) instead of raise ErrorClass, args.
        Calling a dict.iter*() method
        print statement used
        """)
        self._test_output(cmd + ["--py3k", "--rcfile", rcfile],
                          expected_output=expected)

        expected = textwrap.dedent("""
        ************* Module py3k_errors_and_warnings
        Use raise ErrorClass(args) instead of raise ErrorClass, args.
        print statement used
        """)
        self._test_output(cmd + ["--py3k", "-E", "--rcfile", rcfile],
                          expected_output=expected)

        self._test_output(cmd + ["-E", "--py3k", "--rcfile", rcfile],
                          expected_output=expected)

    def test_abbreviations_are_not_supported(self):
        expected = "no such option: --load-plugin"
        self._test_output([".", "--load-plugin"], expected_output=expected)

    def test_enable_all_works(self):
        module = join(HERE, 'data', 'clientmodule_test.py')
        expected = textwrap.dedent("""
        ************* Module data.clientmodule_test
        W: 10, 8: Unused variable 'local_variable' (unused-variable)
        C: 18, 4: Missing method docstring (missing-docstring)
        C: 22, 0: Missing class docstring (missing-docstring)
        """)
        self._test_output([module, "--disable=all", "--enable=all", "-rn"],
                          expected_output=expected)

    def test_wrong_import_position_when_others_disabled(self):
        expected_output = textwrap.dedent('''
        ************* Module wrong_import_position
        C: 11, 0: Import "import os" should be placed at the top of the module (wrong-import-position)
        ''')
        module1 = join(HERE, 'regrtest_data', 'import_something.py')
        module2 = join(HERE, 'regrtest_data', 'wrong_import_position.py')
        args = [module2, module1,
                "--disable=all", "--enable=wrong-import-position",
                "-rn", "-sn"]
        out = six.StringIO()
        self._run_pylint(args, out=out)
        actual_output = out.getvalue().strip()

        to_remove = "No config file found, using default configuration"
        if to_remove in actual_output:
            actual_output = actual_output[len(to_remove):]
        if actual_output.startswith("Using config file "):
            # If ~/.pylintrc is present remove the
            # Using config file...  line
            actual_output = actual_output[actual_output.find("\n"):]
        assert expected_output.strip() == actual_output.strip()

    def test_import_itself_not_accounted_for_relative_imports(self):
        expected = 'Your code has been rated at 10.00/10'
        package = join(HERE, 'regrtest_data', 'dummy')
        self._test_output([package, '--disable=locally-disabled', '-rn'],
                          expected_output=expected)

    def test_reject_empty_indent_strings(self):
        expected = "indent string can't be empty"
        module = join(HERE, 'data', 'clientmodule_test.py')
        self._test_output([module, '--indent-string='],
                          expected_output=expected)

    def test_json_report_when_file_has_syntax_error(self):
        out = six.StringIO()
        module = join(HERE, 'regrtest_data', 'syntax_error.py')
        self._runtest([module], code=2, reporter=JSONReporter(out))
        output = json.loads(out.getvalue())
        assert isinstance(output, list)
        assert len(output) == 1
        assert isinstance(output[0], dict)
        expected = {
            "obj": "",
            "column": 0,
            "line": 1,
            "type": "error",
            "symbol": "syntax-error",
            "module": "syntax_error"
        }
        message = output[0]
        for key, value in expected.items():
            assert key in message
            assert message[key] == value
        assert 'invalid syntax' in message['message'].lower()

    def test_json_report_when_file_is_missing(self):
        out = six.StringIO()
        module = join(HERE, 'regrtest_data', 'totally_missing.py')
        self._runtest([module], code=1, reporter=JSONReporter(out))
        output = json.loads(out.getvalue())
        assert isinstance(output, list)
        assert len(output) == 1
        assert isinstance(output[0], dict)
        expected = {
            "obj": "",
            "column": 0,
            "line": 1,
            "type": "fatal",
            "symbol": "fatal",
            "module": module
        }
        message = output[0]
        for key, value in expected.items():
            assert key in message
            assert message[key] == value
        assert message['message'].startswith("No module named")

    def test_information_category_disabled_by_default(self):
        expected = 'Your code has been rated at 10.00/10'
        path = join(HERE, 'regrtest_data', 'meta.py')
        self._test_output([path], expected_output=expected)

    def test_error_mode_shows_no_score(self):
        expected_output = textwrap.dedent('''
        ************* Module application_crash
        E:  1, 6: Undefined variable 'something_undefined' (undefined-variable)
        ''')
        module = join(HERE, 'regrtest_data', 'application_crash.py')
        self._test_output([module, "-E"], expected_output=expected_output)

    def test_evaluation_score_shown_by_default(self):
        expected_output = 'Your code has been rated at '
        module = join(HERE, 'regrtest_data', 'application_crash.py')
        self._test_output([module], expected_output=expected_output)

    def test_confidence_levels(self):
        expected = 'Your code has been rated at'
        path = join(HERE, 'regrtest_data', 'meta.py')
        self._test_output([path, "--confidence=HIGH,INFERENCE"],
                          expected_output=expected)

    def test_bom_marker(self):
        path = join(HERE, 'regrtest_data', 'meta.py')
        config_path = join(HERE, 'regrtest_data', '.pylintrc')
        expected = 'Your code has been rated at 10.00/10'
        self._test_output([path, "--rcfile=%s" % config_path, "-rn"],
                          expected_output=expected)

    def test_pylintrc_plugin_duplicate_options(self):
        dummy_plugin_path = join(HERE, 'regrtest_data', 'dummy_plugin')
        # Enable --load-plugins=dummy_plugin
        sys.path.append(dummy_plugin_path)
        config_path = join(HERE, 'regrtest_data', 'dummy_plugin.rc')
        expected = (
            ":dummy-message-01 (I9061): *Dummy short desc 01*\n"
            "  Dummy long desc This message belongs to the dummy_plugin checker.\n\n"
            ":dummy-message-02 (I9060): *Dummy short desc 02*\n"
            "  Dummy long desc This message belongs to the dummy_plugin checker.")
        self._test_output(["--rcfile=%s" % config_path,
                           "--help-msg=dummy-message-01,dummy-message-02"],
                          expected_output=expected)
        expected = (
            "[DUMMY_PLUGIN]\n\n# Dummy option 1\ndummy_option_1=dummy value 1\n\n"
            "# Dummy option 2\ndummy_option_2=dummy value 2")
        self._test_output(["--rcfile=%s" % config_path, "--generate-rcfile"],
                          expected_output=expected)
        sys.path.remove(dummy_plugin_path)

    def test_pylintrc_comments_in_values(self):
        path = join(HERE, 'regrtest_data', 'test_pylintrc_comments.py')
        config_path = join(HERE, 'regrtest_data', 'comments_pylintrc')
        expected = textwrap.dedent('''
        ************* Module test_pylintrc_comments
        W:  2, 0: Bad indentation. Found 1 spaces, expected 4 (bad-indentation)
        C:  1, 0: Missing module docstring (missing-docstring)
        C:  1, 0: Missing function docstring (missing-docstring)
        ''')
        self._test_output([path, "--rcfile=%s" % config_path, "-rn"],
                          expected_output=expected)

    def test_no_crash_with_formatting_regex_defaults(self):
        self._runtest(["--ignore-patterns=a"], reporter=TextReporter(six.StringIO()),
                      code=32)

    def test_getdefaultencoding_crashes_with_lc_ctype_utf8(self):
        expected_output = textwrap.dedent('''
        ************* Module application_crash
        E:  1, 6: Undefined variable 'something_undefined' (undefined-variable)
        ''')
        module = join(HERE, 'regrtest_data', 'application_crash.py')
        with _configure_lc_ctype('UTF-8'):
            self._test_output([module, '-E'], expected_output=expected_output)



'''
Complain about multi-argument exception constructors where the first argument
contains a percent sign, thus suggesting a % string formatting was intended
instead.  The same holds for a string containing {...} suggesting str.format()
was intended.
'''

def bad_percent(arg):
    '''Raising a percent-formatted string and an argument'''
    raise KeyError('Bad key: %r', arg)  # [raising-format-tuple]

def good_percent(arg):
    '''Instead of passing multiple arguments, format the message'''
    raise KeyError('Bad key: %r' % arg)

def bad_multiarg(name, value):
    '''Raising a formatted string and multiple additional arguments'''
    raise ValueError('%s measures %.2f', name, value)  # [raising-format-tuple]

def good_multiarg(name, value):
    '''The arguments have to be written as a tuple for formatting'''
    raise ValueError('%s measures %.2f' % (name, value))

def bad_braces(arg):
    '''Curly braces as placeholders'''
    raise KeyError('Bad key: {:r}', arg)  # [raising-format-tuple]

def good_braces(arg):
    '''Call str.format() instead'''
    raise KeyError('Bad key: {:r}'.format(arg))

def bad_multistring(arg):
    '''Multiple adjacent string literals'''
    raise AssertionError(  # [raising-format-tuple]
        'Long message about %s '
        "split over several adjacent literals", arg)

def bad_triplequote(arg):
    '''String literals with triple quotes'''
    raise AssertionError(  # [raising-format-tuple]
        '''Long message about %s
        split over several adjacent literals''', arg)

def bad_unicode(arg):
    '''Unicode string literal'''
    raise ValueError(u'Bad %s', arg)  # [raising-format-tuple]

def raise_something_without_name(arg):
    '''Regression test for nodes without .node attribute'''
    import standard_exceptions  # pylint: disable=import-error
    raise standard_exceptions.MyException(u'An %s', arg)  # [raising-format-tuple]



"""test string format error
"""
# pylint: disable=print-statement,unsupported-binary-operation
from __future__ import print_function

PARG_1 = PARG_2 = PARG_3 = 1

def pprint():
    """Test string format
    """
    print("%s %s" % {'PARG_1': 1, 'PARG_2': 2}) # E1306
    print("%s" % (PARG_1, PARG_2)) # E1305
    print("%(PARG_1)d %d" % {'PARG_1': 1, 'PARG_2': 2}) # E1302
    print("%(PARG_1)d %(PARG_2)d" % {'PARG_1': 1}) # E1304
    print("%(PARG_1)d %(PARG_2)d" % {'PARG_1': 1, 'PARG_2':2, 'PARG_3':3}) #W1301
    print("%(PARG_1)d %(PARG_2)d" % {'PARG_1': 1, 2:3}) # W1300 E1304
    print("%(PARG_1)d %(PARG_2)d" % (2, 3)) # 1303
    print("%(PARG_1)d %(PARG_2)d" % [2, 3]) # 1303
    print("%2z" % PARG_1)
    print("strange format %2" % PARG_2)
    print("works in 3 %a" % 1)



"""check for method without self as first argument
"""
from __future__ import print_function
__revision__ = 0


class Abcd(object):
    """dummy class"""

    def __init__(truc):
        """method without self"""
        print(1)

    def abdc(yoo):
        """another test"""
        print(yoo)
    def edf(self):
        """just another method"""
        print('yapudju in', self)



# pylint: disable=R0903,W0232,missing-docstring
"""test detection of method which could be a function"""

from __future__ import print_function

class Toto(object):
    """bla bal abl"""

    def __init__(self):
        self.aaa = 2

    def regular_method(self):
        """this method is a real method since it access to self"""
        self.function_method()

    def function_method(self): # [no-self-use]
        """this method isn' a real method since it doesn't need self"""
        print('hello')


class Base(object):
    """an abstract class"""

    def __init__(self):
        self.aaa = 2

    def check(self, arg):
        """an abstract method, could not be a function"""
        raise NotImplementedError


class Sub(Base):
    """a concrete class"""

    def check(self, arg):
        """a concrete method, could not be a function since it need
        polymorphism benefits
        """
        return arg == 0

class Super(object):
    """same as before without abstract"""
    attr = 1
    def method(self):
        """regular"""
        print(self.attr)

class Sub1(Super):
    """override method with need for self"""
    def method(self):
        """no i can not be a function"""
        print(42)

    def __len__(self):
        """no i can not be a function"""
        print(42)

    def __cmp__(self, other):
        """no i can not be a function"""
        print(42)

    def __copy__(self):
        return 24

    def __getstate__(self):
        return 42


class Prop(object):

    @property
    def count(self):
        """Don't emit no-self-use for properties.

        They can't be functions and they can be part of an
        API specification.
        """
        return 42
